// copyright (c) 2008 rajh and gregwar. Score stuff
#include <string.h>
#include <sstream>
#include <fstream>
#include <list>
#include <string>
#include <engine/shared/config.h>
#include "gamecontext.h"
#include "score.h"

static LOCK score_lock = 0;

PLAYER_SCORE::PLAYER_SCORE(const char *name, float score, const char *ip, float cp_time[42])
{
	str_copy(this->name, name, sizeof(this->name));
	this->score = score;
	str_copy(this->ip, ip, sizeof(this->ip));
	for(int i = 0; i < 42; i++)
		this->cp_time[i] = cp_time[i];
}

std::list<PLAYER_SCORE> top;

SCORE::SCORE()
{
	if(score_lock == 0)
		score_lock = lock_create();
	load();
}

void SCORE::init(class CGameContext *pGameServer)
{
	m_pGameServer = pGameServer;
	m_pServer = m_pGameServer->Server();
}

std::string save_file()
{
	std::ostringstream oss;
	oss << g_Config.m_SvMap << "_record.dtb";
	return oss.str();
}

static void save_score_thread(void *)
{
	lock_wait(score_lock);
	std::fstream f;
	f.open(save_file().c_str(), std::ios::out);
	if(!f.fail())
	{
		int t = 0;
		for(std::list<PLAYER_SCORE>::iterator i = top.begin(); i != top.end(); i++)
		{
			f << i->name << std::endl << i->score << std::endl  << i->ip << std::endl;
			if(g_Config.m_SvCheckpointSave)
			{
				for(int c = 0; c < 25; c++)
					f << i->cp_time[c+17] << " ";
				f << std::endl;
			}
			t++;
			if(t%50 == 0)
				thread_sleep(1);
		}
	}
	f.close();
	lock_release(score_lock);
}

void SCORE::save()
{
	void *save_thread = thread_create(save_score_thread, 0);
#if defined(CONF_FAMILY_UNIX)
	pthread_detach((pthread_t)save_thread);
#endif
}

void SCORE::load()
{
	lock_wait(score_lock);
	std::fstream f;
	f.open(save_file().c_str(), std::ios::in);
	top.clear();
	while (!f.eof() && !f.fail())
	{
		std::string tmpname, tmpscore, tmpip, tmpcpline;
		std::getline(f, tmpname);
		if(!f.eof() && tmpname != "")
		{
			std::getline(f, tmpscore);
			std::getline(f, tmpip);
			float tmpcptime[42] = {0};
			if(g_Config.m_SvCheckpointSave)
			{
				std::getline(f, tmpcpline);
				char *time = strtok((char*)tmpcpline.c_str(), " ");
				int i = 0;
				while(time != NULL && i < 25)
				{
					tmpcptime[i+17] = atof(time);
					time = strtok(NULL, " ");
					i++;
				}
			}
			top.push_back(*new PLAYER_SCORE(tmpname.c_str(), atof(tmpscore.c_str()), tmpip.c_str(), tmpcptime));
		}
	}
	f.close();
	lock_release(score_lock);
}

PLAYER_SCORE *SCORE::search_score(int id, bool score_ip, int *position)
{
	char ip[16];
	Server()->GetIP(id, ip, sizeof(ip));
	
	int pos = 1;
	for(std::list<PLAYER_SCORE>::iterator i = top.begin(); i != top.end(); i++)
	{
		if(!strcmp(i->ip, ip) && g_Config.m_SvScoreIP && score_ip)
		{
			if(position)
				*position = pos;
			return & (*i);
		}
		pos++;
	}
	
	return search_name(Server()->ClientName(id), position, 0);
}

PLAYER_SCORE *SCORE::search_name(const char *name, int *position, bool nocase)
{
	PLAYER_SCORE *player = 0;
	int pos = 1;
	int found = 0;
	for (std::list<PLAYER_SCORE>::iterator i = top.begin(); i != top.end(); i++)
	{
		if(str_find_nocase(i->name, name))
		{
			if(position)
				*position = pos;
			if(nocase)
			{
				found++;
				player = & (*i);
			}
			if(!strcmp(i->name, name))
				return & (*i);
		}
		pos++;
	}
	if(found > 1)
	{
		if(position)
			*position = -1;
		return 0;
	}
	return player;
}

void SCORE::parsePlayer(int id, float score, float cp_time[42])
{
	const char *name = Server()->ClientName(id);
	char ip[16];
	Server()->GetIP(id, ip, sizeof(ip));
	
	lock_wait(score_lock);
	PLAYER_SCORE *player = search_score(id, 1, 0);
	if(player)
	{
		for(int c = 0; c < 42; c++)
		{
			if(player->cp_time[c] == 0 || player->score > score)
				player->cp_time[c] = cp_time[c];
		}
		if(player->score > score)
		{
			player->score = score;
			str_copy(player->name, name, sizeof(player->name));
		}
	}
	else
		top.push_back(*new PLAYER_SCORE(name, score, ip, cp_time));
	
	top.sort();
	lock_release(score_lock);
	save();
}

void SCORE::initPlayer(int id)
{
	char ip[16];
	Server()->GetIP(id, ip, sizeof(ip));
	PLAYER_SCORE *player = search_score(id, 0, 0);
	if(player)
	{
		lock_wait(score_lock);
		str_copy(player->ip, ip, sizeof(player->ip));
		lock_release(score_lock);
		save();
	}
}

void SCORE::top5_draw(int id, int debut)
{
	int pos = 1;
	char buf[512];
	GameServer()->SendChatTarget(id, "----------- Top 5 -----------");
	for (std::list<PLAYER_SCORE>::iterator i = top.begin(); i != top.end() && pos <= 5+debut; i++)
	{
		if(pos >= debut)
		{
			str_format(buf, sizeof(buf), "%d. %s Time: %d minute(s) %5.3f second(s)", pos, i->name, (int) i->score/60, i->score-((int)i->score/60*60));
			GameServer()->SendChatTarget(id, buf);
		}
		pos++;
	}
	GameServer()->SendChatTarget(id, "-----------------------------");
}
